@@ -299,7 +299,7 @@ def text(s):
     if type(s) is str: return s
     return s.encode('ascii')
 
-def fold(header):
+def fold(header, namelen=0):
     """Fold a header line into multiple crlf-separated lines at column 72.
 
     >>> text(fold(b'foo'))
@@ -318,14 +318,18 @@ def fold(header):
         i += 3
         pre = header[:i]
         header = header[i:]
-    while len(header) > 72:
-        i = header[:72].rfind(b" ")
+    # 72 is the max line length we actually want, but the header field name
+    # has to fit in the first line too (See Debian Bug #863690).
+    maxleng = 72 - namelen
+    while len(header) > maxleng:
+        i = header[:maxleng].rfind(b" ")
         if i == -1:
-            j = 72
+            j = maxleng
         else:
             j = i + 1
         pre += header[:j] + b"\r\n "
         header = header[j:]
+        namelen = 0
     return pre + header
 
 def load_pk_from_dns(name, dnsfunc=get_txt):
@@ -366,9 +370,9 @@ class DomainSigner(object):
         raise ParameterError(
             "Unsupported signature algorithm: "+signature_algorithm)
     self.signature_algorithm = signature_algorithm
-    #: Header fields which should be signed.  Default from RFC4871
+    #: Header fields which should be signed.  Default as suggested by RFC6376
     self.should_sign = set(DKIM.SHOULD)
-    #: Header fields which should not be signed.  The default is from RFC4871.
+    #: Header fields which should not be signed.  The default is from RFC6376.
     #: Attempting to sign these headers results in an exception.
     #: If it is necessary to sign one of these, it must be removed
     #: from this list first.
@@ -385,18 +389,18 @@ class DomainSigner(object):
   #: @since: 0.5
   FROZEN = (b'from',b'date',b'subject')
 
-  #: The rfc4871 recommended header fields to sign
+  #: The rfc6376 recommended header fields to sign
   #: @since: 0.5
   SHOULD = (
     b'sender', b'reply-to', b'subject', b'date', b'message-id', b'to', b'cc',
     b'mime-version', b'content-type', b'content-transfer-encoding',
-    b'content-id', b'content- description', b'resent-date', b'resent-from',
+    b'content-id', b'content-description', b'resent-date', b'resent-from',
     b'resent-sender', b'resent-to', b'resent-cc', b'resent-message-id',
-    b'in-reply-to', 'references', b'list-id', b'list-help', b'list-unsubscribe',
+    b'in-reply-to', b'references', b'list-id', b'list-help', b'list-unsubscribe',
     b'list-subscribe', b'list-post', b'list-owner', b'list-archive'
   )
 
-  #: The rfc4871 recommended header fields not to sign.
+  #: The rfc6376 recommended header fields not to sign.
   #: @since: 0.5
   SHOULD_NOT = (
     b'return-path',b'received',b'comments',b'keywords',b'bcc',b'resent-bcc',
@@ -485,7 +489,7 @@ class DomainSigner(object):
 
     header_value = b"; ".join(b"=".join(x) for x in fields)
     if not standardize:
-      header_value = fold(header_value)
+      header_value = fold(header_value, namelen=len(header_name))
     header_value = RE_BTAG.sub(b'\\1',header_value)
     header = (header_name, b' ' + header_value)
     h = HashThrough(self.hasher())
@@ -508,9 +512,9 @@ class DomainSigner(object):
     idx = [i for i in range(len(fields)) if fields[i][0] == b'b'][0]
     fields[idx] = (b'b', base64.b64encode(bytes(sig2)))
     header_value = b"; ".join(b"=".join(x) for x in fields) + b"\r\n"
-    
+
     if not standardize:
-      header_value = fold(header_value)
+      header_value = fold(header_value, namelen=len(header_name))
 
     return header_value
 
@@ -656,7 +660,7 @@ class DKIM(DomainSigner):
         (b'c', canon_policy.to_c_value()),
         (b'd', domain),
         (b'i', identity or b"@"+domain),
-        length and (b'l', len(body)),
+        length and (b'l', str(len(body)).encode('ascii')),
         (b'q', b"dns/txt"),
         (b's', selector),
         (b't', str(int(time.time())).encode('ascii')),
@@ -810,7 +814,7 @@ class ARC(DomainSigner):
     arc_headers = [y for x,y in arc_headers_w_instance]
 
     # Compute ARC-Authentication-Results
-    aar_value = b"i=%d; %s" % (instance, auth_results)
+    aar_value = ("i=%d; " % instance).encode('utf-8') + auth_results
     if aar_value[-1] != b'\n': aar_value += b'\r\n'
 
     new_arc_set.append(b"ARC-Authentication-Results: " + aar_value)
@@ -1080,4 +1084,3 @@ def arc_verify(message, logger=None, dnsfunc=get_txt, minkey=1024):
         if logger is not None:
             logger.error("%s" % x)
         return CV_Fail, [], "%s" % x
-
